local dimensions= require("dimensions")
local images= require("images")
local key_config= require("key_config")
local sprite_oneesan= require("sprite")
local entity_oneesan= require("entity")
local clutter= require("clutter")

local alien_pool= {}
local explosion_pool= {}
local explosion_life= .125

local max_formation_width= 20
local formation_x_spacing= 20
local formation_y_spacing= 20
local formation_speed= 50
local formation_min_x= 20
local formation_max_x= 380
local formation_phase= 1
local formation_entity= {}
local next_formation_slot= 0

local formation_entity= {}

local function explode_entity(curr_time, ent)
	explosion_pool[#explosion_pool+1]= {
		death_time= explosion_life + curr_time,
		image= images.explosion,
		sprite= sprite_oneesan.create_sprite(
			curr_time, #images.explosion.frames, explosion_life),
		entity= entity_oneesan.create_entity(
			"explosions", curr_time, ent.x, ent.y, 0, 0,
			ent.speed_x, ent.speed_y),
	}
end

local function update(curr_time)
	for e= #explosion_pool, 1, -1 do
		local exp= explosion_pool[e]
		if curr_time >= exp.death_time then
			exp.entity:destroy("explosions")
			exp.sprite:destroy()
			table.remove(explosion_pool, e)
		end
	end
end

local function hit_torps(curr_time, torp_clutter)
	for a= #alien_pool, 1, -1 do
		local alien= alien_pool[a]
		local collision_list= torp_clutter:find_collisions(curr_time, alien.entity)
		if #collision_list > 0 then
			local torps_to_hit= math.min(#collision_list, alien.hp)
			alien.hp= alien.hp - torps_to_hit
			for t= 1, #collision_list do
				local torp= collision_list[t]
				torp.death_time= curr_time
			end
			if alien.hp <= 0 then
				explode_entity(curr_time, alien.entity)
				alien.entity:destroy("aliens")
				table.remove(alien_pool, a)
			end
		end
	end
end

local function draw()
	local acx= images.aliens[1].center.x
	local acy= images.aliens[1].center.y
	for a= 1, #alien_pool do
		local alien= alien_pool[a]
		love.graphics.draw(
			alien.image.image, alien.image.frames[alien.frame],
			alien.entity.x - acx, alien.entity.y - acy)
	end
	if #explosion_pool > 0 then
		local ecx= images.explosion.center.x
		local ecy= images.explosion.center.y
		for e= 1, #explosion_pool do
			local exp= explosion_pool[e]
			love.graphics.draw(
				exp.image.image, exp.image.frames[exp.sprite.curr_frame],
				exp.entity.x - ecx, exp.entity.y - ecy)
		end
	end
end

local function spawn(number, level, hp)
	local image= images.aliens[level]
	local w= image.center.w
	local h= image.center.h
	local curr_time= love.timer.getTime()
	for a= 1, number do
		local x= dimensions.space_width * love.math.random()
		local y= dimensions.space_height * love.math.random()
		local xs= love.math.random() * 100
		local ys= love.math.random() * 100
		alien_pool[#alien_pool+1]= {
			image= image, frame= 1, hp= hp,
			entity= entity_oneesan.create_entity(
				"aliens", curr_time, x, y, w, h, xs, ys),
		}
	end
end

local function begin()
	alien_pool= {}
end

local function end_gameplay()
	alien_pool= nil
end

key_config.register_section(
	"gameplay", {
		{name= "spawn a", keys= {"a"}, press= function() spawn(10, 1, 1) end},
		{name= "spawn s", keys= {"s"}, press= function() spawn(10, 2, 1) end},
		{name= "spawn d", keys= {"d"}, press= function() spawn(10, 3, 5) end},
		{name= "spawn f", keys= {"f"}, press= function() spawn(10, 4, 5) end},
})

return {
	update= update,
	draw= draw,
	hit_torps= hit_torps,
	spawn= spawn,
	begin= begin,
	end_gameplay= end_gameplay,
}
