local dimensions= require("dimensions")
local images= require("images")
local key_config= require("key_config")
local sprite_oneesan= require("sprite")
local entity_oneesan= require("entity")
local player= require("player")
local aliens= require("aliens")

local return_to_pregame

local pause_menu_start_y= 100
local pause_menu_row_spacing= 20
local pause_menu_key_x= 50
local pause_menu_name_x= 100

local pause_start_time= 0

local function update(delta)
	local curr_time= love.timer.getTime()
	aliens.update(curr_time)
	aliens.hit_torps(curr_time, player.get_torps())
	player.update(curr_time)
	sprite_oneesan.update(curr_time)
	entity_oneesan.update(curr_time)
end

local function draw()
	love.graphics.push()
	love.graphics.translate(0, dimensions.status_height)
	love.graphics.setColor(255, 255, 255)
	aliens.draw()
	player.draw()
	love.graphics.pop()
end

local function pause_update(delta)
	local curr_time= love.timer.getTime()
	sprite_oneesan.update(curr_time)
end

local function pause_draw()
	draw()
	local section_info= key_config.get_section_info("pause")
	for i= 1, #section_info do
		local entry= section_info[i]
		local row_y= pause_menu_start_y + ((i-1) * pause_menu_row_spacing)
		local key= entry.keys[1]
		if key then
			love.graphics.print(key, pause_menu_key_x, row_y)
		end
		love.graphics.print(entry.name, pause_menu_name_x, row_y)
	end
end

local function pause_mousepressed(x, y, button)
	local row_id= math.ceil((y - pause_menu_start_y) / pause_menu_row_spacing)
	if row_id < 1 then return end
	local section_info= key_config.get_section_info("pause")
	if row_id > #section_info then return end
	if x >= pause_menu_name_x and x < pause_menu_name_x + 100 then
		local row= section_info[row_id]
		if row.press then
			row.press()
		elseif row.release then
			row.release()
		end
	end
end

local function begin(modore)
	return_to_pregame= modore
	entity_oneesan.create_pool("player")
	entity_oneesan.create_pool("mtorps")
	entity_oneesan.create_pool("etorps")
	entity_oneesan.create_pool("aliens")
	entity_oneesan.create_pool("prizes")
	entity_oneesan.create_pool("explosions")
	player.begin()
	love.update= update
	love.draw= draw
	key_config.set_active_section("gameplay")
end

local function pause()
	pause_start_time= love.timer.getTime()
	key_config.set_active_section("pause")
	love.update= pause_update
	love.draw= pause_draw
	love.mousepressed= pause_mousepressed
end

local function resume()
	key_config.set_active_section("gameplay")
	local curr_time= love.timer.getTime()
	local pause_time= curr_time - pause_start_time
	entity_oneesan.unpause(curr_time)
	player.unpause(pause_time)
	love.update= update
	love.draw= draw
	love.mousepressed= nil
end

local function end_gameplay()
	player.end_gameplay()
	sprite_oneesan.clear()
	entity_oneesan.clear()
	return_to_pregame()
end

local function key_config_return()
	sprite_oneesan.pop_stack()
	entity_oneesan.pop_stack()
	pause()
end

local function open_key_config()
	sprite_oneesan.push_stack()
	entity_oneesan.push_stack()
	key_config.begin(key_config_return)
end

key_config.register_section(
	"gameplay", {
		{name= "pause", keys= {"escape"}, press= pause},
})

key_config.register_section(
	"pause", {
		{name= "exit", keys= {"q"}, release= end_gameplay},
		{name= "resume", keys= {"escape"}, press= resume},
		{name= "key_config", keys= {"k"}, press= open_key_config},
})

return {
	begin= begin,
}
